package sun.security.krb5;

import java.security.AccessController;
import java.security.PrivilegedAction;
import sun.security.krb5.internal.Krb5;
import java.io.IOException;
import java.util.Iterator;
import java.util.Vector;
import java.util.Collection;
import java.util.Hashtable;

public class SCDynamicStoreConfig
{
    private static boolean DEBUG;
    
    public SCDynamicStoreConfig() {
    }
    
    private static native void installNotificationCallback();
    
    private static native Hashtable<String, Object> getKerberosConfig();
    
    private static Vector<String> unwrapHost(Collection<Hashtable<String, String>> collection) {
        Vector<Object> vector = new Vector();
        Iterator<Object> iterator = collection.iterator();
        while (iterator.hasNext()) {
            vector.add(((Hashtable)iterator.next()).get("host"));
        }
        return vector;
    }
    
    private static Hashtable<String, Object> convertRealmConfigs(Hashtable<String, ?> hashtable) {
        Hashtable<String, Object> hashtable2 = new Hashtable();
        for (String s : hashtable.keySet()) {
            Hashtable<Object, Object> hashtable3 = (Hashtable)hashtable.get(s);
            Hashtable value = new Hashtable();
            Collection<Hashtable<String, String>> collection = (Collection)hashtable3.get("kdc");
            if (collection != null) {
                value.put("kdc", unwrapHost(collection));
            }
            Collection<Hashtable<String, String>> collection2 = (Collection)hashtable3.get("kadmin");
            if (collection2 != null) {
                value.put("admin_server", unwrapHost(collection2));
            }
            hashtable2.put(s, value);
        }
        return hashtable2;
    }
    
    public static Hashtable<String, Object> getConfig() throws IOException {
        Hashtable<String, Object> kerberosConfig = getKerberosConfig();
        if (kerberosConfig == null) {
            throw new IOException("Could not load configuration from SCDynamicStore");
        }
        if (SCDynamicStoreConfig.DEBUG) {
            System.out.println("Raw map from JNI: " + kerberosConfig);
        }
        return convertNativeConfig(kerberosConfig);
    }
    
    private static Hashtable<String, Object> convertNativeConfig(Hashtable<String, Object> obj) {
        Hashtable<String, Object> hashtable = (Hashtable)obj.get("realms");
        if (hashtable != null) {
            obj.remove("realms");
            obj.put("realms", convertRealmConfigs(hashtable));
        }
        WrapAllStringInVector(obj);
        if (SCDynamicStoreConfig.DEBUG) {
            System.out.println("stanzaTable : " + obj);
            return obj;
        }
        return obj;
    }
    
    private static void WrapAllStringInVector(Hashtable<String, Object> hashtable) {
        for (String s : hashtable.keySet()) {
            Object value = hashtable.get(s);
            if ((value instanceof Hashtable)) {
                WrapAllStringInVector((Hashtable)value);
            }
            else if ((value instanceof String)) {
                Vector value2 = new Vector();
                value2.add((String)value);
                hashtable.put(s, value2);
            }
            else {
                continue;
            }
        }
    }
    
    static {
        SCDynamicStoreConfig.DEBUG = Krb5.DEBUG;
        if ((boolean)AccessController.doPrivileged(new PrivilegedAction<Boolean>() {
            @Override
            public Boolean run() {
                if (System.getProperty("os.name").contains("OS X")) {
                    System.loadLibrary("osx");
                    return Boolean.valueOf(true);
                }
                return Boolean.valueOf(false);
            }
        })) {
            installNotificationCallback();
        }
    }
}
